package com.huimv.eartag2.manage.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.*;
import com.huimv.eartag2.common.dao.repo.*;
//import com.huimv.eartag2.common.service.IDevice;
import com.huimv.eartag2.common.utils.*;
import com.huimv.eartag2.manage.service.ICacheService;
import com.huimv.eartag2.manage.service.IDeviceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.*;
import org.springframework.data.jpa.domain.Specification;

import javax.persistence.criteria.*;

import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceServiceImpl implements IDeviceService {
    @Value("${device.online.access_mode}")
    private String deviceOnlineAccessMode;
    @Autowired
    private ICacheService cacheService;
    @Autowired
    private EartarFarmAllStatusRepo farmAllStatusRepo;
    @Autowired
    private EartagDeviceRegisterRepo deviceRegisterRepo;
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Autowired
    private EartagDeviceOnlineRepo deviceOnlineRepo;
    @Autowired
    private EartagEartagOnlineRepo eartagOnlineRepo;
    @Autowired
    private EartagDeviceEartagCountRepo deviceEartagCountRepo;
    @Autowired
    private EartagEnvRepo eartagEnvRepo;
    @Autowired
    private EartagHeartbeatRepo heartbeatRepo;


    /**
     * @Method      : listEartagOnlineAllStatus
     * @Description : 
     * @Params      : [farmId, days]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/10       
     * @Time        : 20:52
     */
    @Override
    public Result listEartagOnlineAllStatus(String farmId, Integer days) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String pastDate = dateUtil.getPastDate(days);
        String todayDate = dateUtil.getTodayDateText();
        List<EartarFarmAllStatusEntity> FarmAllStatusEntityList = farmAllStatusRepo.getOneByFarmIdAndCreateDateAndPastDate(farmId, pastDate, todayDate);
        JSONArray dataJa = new JSONArray();
        for (EartarFarmAllStatusEntity farmAllStatusEntity : FarmAllStatusEntityList) {
            JSONObject newJo = new JSONObject();
            dataJa.add(newJo);
            newJo.put("time", dateUtil.formatDateText(farmAllStatusEntity.getCreateDate()));
            newJo.put("value", farmAllStatusEntity.getEartagOnline());
            newJo.put("total", farmAllStatusEntity.getEartagTotal());
        }
        return new Result(ResultCode.SUCCESS, dataJa.toJSONString());
    }

    /**
     * @Method      : listDeviceEnvtempByDeviceCode
     * @Description : 
     * @Params      : [deviceCode]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8       
     * @Time        : 20:55
     */
    @Override
    public Result listDeviceEnvtempByDeviceCode(String deviceCode) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        //
        List<EartagEnvEntity> eartagEnvEntityList = eartagEnvRepo.getDeviceEnvtempByDeviceCode(deviceCode,todayDateText);
        if(eartagEnvEntityList.size()==0){
            return new Result(BizConst.CODE_ENVTEMP_FLOW_NO_EXIST,BizConst.MSG_ENVTEMP_FLOW_NO_EXIST,false);
        }else{
            JSONArray dataJa = new JSONArray();
            for(EartagEnvEntity eartagEnvEntity:eartagEnvEntityList){
                JSONObject eartagEnvJo = JSONUtil.convertEntityToJSONObject(eartagEnvEntity);
                eartagEnvJo.put("addTime",du.formatDatetimeText(eartagEnvJo.getDate("addTime")));
                dataJa.add(eartagEnvJo);
            }
            return new Result(ResultCode.SUCCESS,dataJa);
        }
    }

    /**
     * @Method      : listDeviceEartagOnlineCountByDeviceCodeInPager
     * @Description : 显示设备关联耳标统计（带分页）
     * @Params      : [deviceCode, earmark, addDate, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/9       
     * @Time        : 20:08
     */
    @Override
    public Result listDeviceEartagOnlineCountByDeviceCodeInPager(String deviceCode, String earmark, String addDate, Integer pageNo, Integer pageSize) throws ParseException {
        //
        Specification<EartagDeviceEartagCountEntity> sf = (Specification<EartagDeviceEartagCountEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != earmark && earmark.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("earmark").as(String.class), earmark));
            }
            //
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            //
            if (null != addDate && addDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.equal(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(addDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, deviceEartagCountRepo.findAll(sf, pageable));
    }

    @Override
    public Result listHeartbeatByDeviceCodeAndDateInPager(String deviceCode, String addDate, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagHeartbeatEntity> sf = (Specification<EartagHeartbeatEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            //
            if (null != addDate && addDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.equal(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(addDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, heartbeatRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listDeviceEnvtempByDeviceCodeInPager
     * @Description : 分页查询设备温度
     * @Params      : [deviceCode, addDate, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/9       
     * @Time        : 21:27
     */
    @Override
    public Result listDeviceEnvtempByDeviceCodeInPager(String deviceCode, String addDate, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagEnvEntity> sf = (Specification<EartagEnvEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            //
            if (null != addDate && addDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.equal(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(addDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, eartagEnvRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listDeviceRegisterInPager
     * @Description : 列表查询分页信息（分页）
     * @Params      : [farmId, deviceCode, alias, startDate, endDate, activeStatus, liveStatus, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/9       
     * @Time        : 22:28
     */
    @Override
    public Result listDeviceRegisterInPager(String farmId, String deviceCode, String alias, String startDate, String endDate, Integer activeStatus, Integer liveStatus, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagDeviceRegisterEntity> sf = (Specification<EartagDeviceRegisterEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //farmId
            if (null != farmId && farmId.trim().length()>0) {
                System.out.println("### farmId>>"+farmId);
                predList.add(criteriaBuilder.equal(root.get("farmId").as(String.class), farmId));
            }
            //deviceCode
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            //alias
            if (null != alias) {
                predList.add(criteriaBuilder.like(root.get("deviceAlias").as(String.class), alias));
            }
            //startDate
            if (null != startDate && startDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.greaterThanOrEqualTo(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(startDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            //endDate
            if (null != endDate && endDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.lessThanOrEqualTo(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(endDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            //activeStatus
            if (null != activeStatus) {
                predList.add(criteriaBuilder.equal(root.get("activeStatus").as(Integer.class), activeStatus));
            }
            //liveStatus
            if (null != liveStatus) {
                predList.add(criteriaBuilder.equal(root.get("deviceStatus").as(Integer.class), liveStatus));
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, deviceRegisterRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listOnlineDevice
     * @Description : 
     * @Params      : [farmId, deviceCode, alias, addDate, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8       
     * @Time        : 20:55
     */
    @Override
    public Result listOnlineDevice(String farmId, String deviceCode, String alias, String addDate, Integer pageNo, Integer pageSize) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String todayDate = dateUtil.getTodayDateText();
        //* 这里提供2种方式：1、从数据库读取 2、从缓存中取数（后期优化)，但是同时提供2种方式都可用，防止缓存方式失效 */
        System.out.println("deviceOnlineAccessMode>>" + deviceOnlineAccessMode);
        if (deviceOnlineAccessMode.trim().equalsIgnoreCase("mysql")) {
            //
            return listDeviceOnlineFromMySQL(farmId,addDate , deviceCode, alias, pageNo, pageSize);
        } else {
            //
            return listDeviceOnlineFromCache(farmId, todayDate);
        }
    }

    //分页查询
    private Result listDeviceOnlineFromMySQL(String farmId, String addDate, String deviceCode, String alias, Integer pageNo, Integer pageSize) {
//        //
//        List<EartagDeviceOnlineEntity> deviceOnlineEntityList = deviceOnlineRepo.getDeviceOnlineByFarmIdAndCreateDate(farmId,addDate);
//        if(deviceOnlineEntityList.size()==0){
//            return new Result(BizConst.CODE_DEVICE_ONLINE_NO_EXIST,BizConst.MSG_DEVICE_ONLINE_NO_EXIST,false);
//        }else{
//            return new Result(ResultCode.SUCCESS,JSONObject.toJSONString(deviceOnlineEntityList));
//        }
        DateUtil dateUtil = new DateUtil();
        //
        Specification<EartagDeviceOnlineEntity> sf = (Specification<EartagDeviceOnlineEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != farmId) {
                predList.add(criteriaBuilder.equal(root.get("farmId").as(String.class), farmId));
            }
            if (null != deviceCode) {
                predList.add(criteriaBuilder.equal(root.get("deviceCode").as(String.class), deviceCode));
            }
            if (null != addDate) {
                try {
                    predList.add(criteriaBuilder.equal(root.get("addDate").as(Date.class), new Date(dateUtil.parseDateTextToLong(addDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            if (null != alias) {
                predList.add(criteriaBuilder.equal(root.get("alias").as(String.class), alias));
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, deviceOnlineRepo.findAll(sf, pageable));
    }

    /**
     * @Method : listDeviceEartagOnlineCountByDeviceCode
     * @Description :
     * @Params : [deviceCode]
     * @Return : com.huimv.eartag2.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2022/3/7
     * @Time : 17:13
     */
    @Override
    public Result listDeviceEartagOnlineCountByDeviceCode(String deviceCode) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        //
        List<EartagDeviceEartagCountEntity> deviceEartagCountEntityList = deviceEartagCountRepo.getOnlineEartagByDeviceCode(deviceCode, todayDateText);
        if (deviceEartagCountEntityList.size() == 0) {
            return new Result(BizConst.CODE_DEVICE_EARTAG_COUNT_NO_EXIST, BizConst.MSG_DEVICE_EARTAG_COUNT_NO_EXIST, false);
        } else {
            return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(deviceEartagCountEntityList));
        }
    }

    /**
     * @Method : listDeviceOnlineStatusByDeviceCode
     * @Description : 列表在线设备关联在线耳标统计
     * @Params : [deviceCode, pastDays]
     * @Return : com.huimv.eartag2.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2022/3/7
     * @Time : 17:13
     */
    @Override
    public Result listDeviceOnlineStatusByDeviceCode(String deviceCode, Integer pastDays) throws ParseException {
        DateUtil du = new DateUtil();
        DateUtil dateUtil = new DateUtil();
        String pastDate = dateUtil.getPastDate(pastDays);
        String todayDate = dateUtil.getTodayDateText();
        //
        List<EartagDeviceOnlineEntity> deviceOnlineEntityList = deviceOnlineRepo.getDeviceOnlineByDeviceCode(deviceCode, pastDate, todayDate);
        if (deviceOnlineEntityList.size() > 0) {
            List<String> pastDateList = du.getPastIntervalsASC(pastDays);
            JSONArray dataJa = new JSONArray();
            for (String pastDayDate : pastDateList) {
                JSONObject newJo = new JSONObject();
                dataJa.add(newJo);
                newJo.put("time", pastDayDate);
                boolean ieFound = false;
                for (EartagDeviceOnlineEntity deviceOnlineEntity : deviceOnlineEntityList) {
                    int compareTo = du.parseDate(pastDayDate).compareTo(deviceOnlineEntity.getAddDate());
                    if (compareTo == 0) {
                        newJo.put("value", 1);
                        ieFound = true;
                        break;
                    }
                }
                if (!ieFound) {
                    newJo.put("value", 0);
                }
            }
            return new Result(ResultCode.SUCCESS, dataJa);
        } else {
            return new Result(BizConst.CODE_DEVICE_ONLINE_NO_EXIST, BizConst.MSG_DEVICE_ONLINE_NO_EXIST, false);
        }
    }

    /**
     * @Method : listDeviceOnlineByDeviceCode
     * @Description : 查询单个设备N天在线记录
     * @Params : [deviceCode]
     * @Return : com.huimv.eartag2.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2022/3/7
     * @Time : 14:08
     */
    @Override
    public Result listDeviceOnlineCountByDeviceCode(String deviceCode, Integer pastDays, Integer showMode) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String pastDate = dateUtil.getPastDate(pastDays);
        String todayDate = dateUtil.getTodayDateText();
        //
        List<EartagDeviceOnlineEntity> deviceOnlineEntityList = deviceOnlineRepo.getDeviceOnlineByDeviceCode(deviceCode, pastDate, todayDate);
        if (deviceOnlineEntityList.size() > 0) {
            if (showMode == 1) {
                return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(deviceOnlineEntityList));
            } else {
                JSONArray dataJa = new JSONArray();
                for (EartagDeviceOnlineEntity deviceOnlineEntity : deviceOnlineEntityList) {
                    JSONObject newJo = new JSONObject();
                    dataJa.add(newJo);
                    newJo.put("time", dateUtil.formatDateText(deviceOnlineEntity.getAddDate()));
                    newJo.put("value", deviceOnlineEntity.getEartagTotal());
                }
                return new Result(ResultCode.SUCCESS, dataJa.toJSONString());
            }
        } else {
            return new Result(BizConst.CODE_DEVICE_ONLINE_NO_EXIST, BizConst.MSG_DEVICE_ONLINE_NO_EXIST, false);
        }
    }

    /**
     * @Method : getDeviceRegisterByDeviceCode
     * @Description : 通过设备编号显示设备注册信息
     * @Params : [deviceCode]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/3/7
     * @Time : 13:55
     */
    @Override
    public Result getDeviceRegisterByDeviceCode(String deviceCode) {
        //
        EartagDeviceRegisterEntity deviceRegisterEntity = deviceRegisterRepo.getDeviceRegister(deviceCode);
        if (deviceRegisterEntity == null) {
            log.error(BizConst.MSG_DEVICE_REGISTER_NO_EXIST);
            return new Result(BizConst.CODE_DEVICE_REGISTER_NO_EXIST, BizConst.MSG_DEVICE_REGISTER_NO_EXIST, false);
        } else {
            return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(deviceRegisterEntity));
        }
    }

    /**
     * @Method : listOnlineDevice
     * @Description : 显示在线设备记录
     * @Params : [farmId]
     * @Return : com.huimv.eartag2.common.utils.Result
     * @Author : ZhuoNing
     * @Date : 2022/3/7
     * @Time : 13:21
     */
    @Override
    public Result listOnlineDevice(String farmId) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String todayDate = dateUtil.getTodayDateText();
        //* 这里提供2种方式：1、从数据库读取 2、从缓存中取数（后期优化)，但是同时提供2种方式都可用，防止缓存方式失效 */
        System.out.println("deviceOnlineAccessMode>>" + deviceOnlineAccessMode);
        if (deviceOnlineAccessMode.trim().equalsIgnoreCase("mysql")) {
            //
            return listDeviceOnlineFromMySQL(farmId, todayDate);
        } else {
            //
            return listDeviceOnlineFromCache(farmId, todayDate);
        }
    }


    //从缓存读取在线记录（后期优化）
    private Result listDeviceOnlineFromCache(String farmId, String todayDate) {


        return null;
    }

    //从数据库读取在线记录
    private Result listDeviceOnlineFromMySQL(String farmId, String todayDate) throws ParseException {
        DateUtil du = new DateUtil();
        //
        List<EartagDeviceOnlineEntity> deviceOnlineEntityList = deviceOnlineRepo.getDeviceOnlineByFarmIdAndCreateDate(farmId, todayDate);
        if (deviceOnlineEntityList.size() == 0) {
            return new Result(BizConst.CODE_DEVICE_ONLINE_NO_EXIST, BizConst.MSG_DEVICE_ONLINE_NO_EXIST, false);
        } else {
//            for(EartagDeviceOnlineEntity deviceOnlineEntity:deviceOnlineEntityList){
//                deviceOnlineEntity.setHeartbeatFirstTime(du.formatDatetimeText());
//            }
            return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(deviceOnlineEntityList));
        }
    }

    @Override
    public Result getNowAllStatus(String farmId) throws ParseException {
        //#读取设备和耳标总状态缓存;
        //      总状态缓存存在：从缓存读取数据返回；
        //      总状态缓存不存在：从数据库读取记录：
        //              {
        //              数据库记录存在：读取记录返回，并初始化缓存；
        //              数据库记录不存在：则构建数据库记录，并初始化缓存，返回结果；
        //              }

        //{从总状态缓存钟读取缓存数据}
        DateUtil dateUtil = new DateUtil();
        Map farmAllStatusMap = cacheService.getFarmAllStatusByFarmId(farmId);
        if (farmAllStatusMap.size() > 0) {
            log.info("返回缓存数据.");
            return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(farmAllStatusMap));
        } else {
            //#从数据库读取总状态表
            String todayDateText = dateUtil.getTodayDateText();
            Date todayDate = dateUtil.getTodayMySQLDate();
            Timestamp timestamp = dateUtil.getTimestamp();
            EartarFarmAllStatusEntity farmAllStatusEntity = farmAllStatusRepo.getOneByFarmIdAndCreateDate(farmId, todayDateText);
            if (farmAllStatusEntity != null) {
                //#初始化总状态缓存
                cacheService.putFarmAllStatusCache(farmId, farmAllStatusEntity);
                return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(farmAllStatusEntity));
            } else {
                //*构建总状态数据 */
                int deviceTotal = getDeviceCancelTotalFromDeviceRegister(farmId);
                int eartagTotal = getEartagRegTotalByFarmId(farmId);
                int deviceOnlineTotal = cacheService.getDeviceOnlineTotal(farmId, todayDateText);
                int eartagOnlineTotal = cacheService.getEartagOnlineTotal(farmId, todayDateText);
                int deviceOffline = deviceTotal - deviceOnlineTotal;
                BigDecimal rateBd = new BigDecimal(deviceOnlineTotal).divide(new BigDecimal(deviceTotal), 1, BigDecimal.ROUND_HALF_UP);
                EartarFarmAllStatusEntity newFarmAllStatusEntity = new EartarFarmAllStatusEntity();
                newFarmAllStatusEntity.setDeviceTotal(deviceTotal);
                newFarmAllStatusEntity.setDeviceOnline(deviceOnlineTotal);
                newFarmAllStatusEntity.setDeviceOffline(deviceOffline);
                newFarmAllStatusEntity.setDeviceRate(Float.parseFloat(rateBd.toString()));
                newFarmAllStatusEntity.setEartagTotal(eartagTotal);
                newFarmAllStatusEntity.setEartagOnline(eartagOnlineTotal);
                newFarmAllStatusEntity.setFarmId(farmId);
                newFarmAllStatusEntity.setUpdateTime(timestamp);
                newFarmAllStatusEntity.setCreateDate(todayDate);
                farmAllStatusRepo.saveAndFlush(newFarmAllStatusEntity);
                //*初始化总状态缓存*/
                cacheService.putFarmAllStatusCache(farmId, newFarmAllStatusEntity);
                return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(newFarmAllStatusEntity));
            }
        }

    }

    @Override
    public Result listManyDaysAllStatus(String farmId, Integer days) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String pastDate = dateUtil.getPastDate(days);
        String todayDate = dateUtil.getTodayDateText();
        List<EartarFarmAllStatusEntity> FarmAllStatusEntityList = farmAllStatusRepo.getOneByFarmIdAndCreateDateAndPastDate(farmId, pastDate, todayDate);
        JSONArray dataJa = new JSONArray();
        for (EartarFarmAllStatusEntity farmAllStatusEntity : FarmAllStatusEntityList) {
            JSONObject newJo = new JSONObject();
            dataJa.add(newJo);
            newJo.put("time", dateUtil.formatDateText(farmAllStatusEntity.getCreateDate()));
            newJo.put("value", farmAllStatusEntity.getDeviceOnline());
            newJo.put("total", farmAllStatusEntity.getDeviceTotal());
            newJo.put("rate", farmAllStatusEntity.getDeviceRate());
            newJo.put("offline", farmAllStatusEntity.getDeviceOffline());
        }
        return new Result(ResultCode.SUCCESS, dataJa.toJSONString());
    }

    //计算注册设备数量
    private int getDeviceCancelTotalFromDeviceRegister(String farmId) {
        List<Object[]> deviceRegList = deviceRegisterRepo.getDeviceCountByFarmId(farmId);
        Object[] deviceRegObj = (Object[]) deviceRegList.get(0);
        //设备总数
        return Integer.parseInt(deviceRegObj[0].toString());
    }

    //计算注册耳标数量
    public Integer getEartagRegTotalByFarmId(String farmId) {
        int eartagTotal = 0;
        List<Object[]> eartagRegList = eartagRegisterRepo.getEartagCountByFarmId(farmId);
        Object[] eartagRegObj = (Object[]) eartagRegList.get(0);
        //处于工作状态的耳标总数
        return Integer.parseInt(eartagRegObj[0].toString());
    }
}
