package com.huimv.eartag2.manage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.dao.entity.*;
import com.huimv.eartag2.common.dao.repo.EartagDataRepo;
import com.huimv.eartag2.common.dao.repo.EartagEartagOnlineRepo;
import com.huimv.eartag2.common.dao.repo.EartagEartagRegisterRepo;
import com.huimv.eartag2.common.utils.*;
import com.huimv.eartag2.manage.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.Order;
import javax.persistence.criteria.Predicate;
import java.sql.Date;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EartagServiceImpl implements IEartagService {
    @Autowired
    private EartagEartagRegisterRepo eartagRegisterRepo;
    @Autowired
    private EartagDataRepo eartagDataRepo;
    @Autowired
    private EartagEartagOnlineRepo eartagOnlineRepo;

    /**
     * @Method      : listEartagOnlineStatusByEarmarkAndDate
     * @Description : 显示耳标在线统计
     * @Params      : [earmark, pastDays]
     * @Return      : com.huimv.eartag2.common.utils.Result
     *
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8
     * @Time        : 18:30
     */
    @Override
    public Result listEartagOnlineStatusByEarmarkAndDate(String earmark, Integer pastDays) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        String startDateText = du.getPastDate(pastDays);
        log.info("<listEartagOnlineStatusByEarmarkAndDate> startDateText>>"+startDateText);
        log.info("<listEartagOnlineStatusByEarmarkAndDate> todayDateText>>"+todayDateText);
        //读取耳标在线统计记录
        List<EartagEartagOnlineEntity> eartagOnlineEntityList = eartagOnlineRepo.getEartagOnlineByEarmarkAndDate(earmark,startDateText,todayDateText);
        if(eartagOnlineEntityList.size()>0){
            JSONArray dataJa = new JSONArray();
            List<String> pastDateList = du.getPastIntervalsASC(pastDays);
            for (String pastDayDate : pastDateList) {
                JSONObject newJo = new JSONObject();
                dataJa.add(newJo);
                newJo.put("time", pastDayDate);
                boolean ieFound = false;
                for (EartagEartagOnlineEntity eartagOnlineEntity:eartagOnlineEntityList) {
                    int compareTo = du.parseDate(pastDayDate).compareTo(eartagOnlineEntity.getAddDate());
                    if (compareTo == 0) {
                        newJo.put("value", 1);
                        ieFound = true;
                        break;
                    }
                }
                if (!ieFound) {
                    newJo.put("value", 0);
                }
            }
//            for(EartagEartagOnlineEntity eartagOnlineEntity:eartagOnlineEntityList){
//                JSONObject eartagOnlineJo = JSONUtil.convertEntityToJSONObject(eartagOnlineEntity);
//                eartagOnlineJo.put("firstTime",du.formatDatetimeText(eartagOnlineJo.getDate("firstTime")));
//                eartagOnlineJo.put("lastTime",du.formatDatetimeText(eartagOnlineJo.getDate("lastTime")));
//                eartagOnlineJo.put("addDate",du.formatDateText(eartagOnlineJo.getDate("addDate")));
//                dataJa.add(eartagOnlineJo);
//            }
            return new Result(ResultCode.SUCCESS,dataJa);
        }else{
            return new Result(BizConst.CODE_EARTAG_ONLINE_NO_EXIST,BizConst.MSG_EARTAG_ONLINE_NO_EXIST,false);
        }
    }

    /**
     * @Method      : getEartagRegisterByEarmarkInPager
     * @Description : 分页查询耳标流水
     * @Params      : [earmark, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8       
     * @Time        : 20:19
     */
    @Override
    public Result listEartagFlowByEarmarkAndTodayInPager(String earmark, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagDataEntity> sf = (Specification<EartagDataEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            if (null != earmark) {
                predList.add(criteriaBuilder.equal(root.get("earmark").as(String.class), earmark));
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, eartagDataRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listEartagFlowByDeviceCodeAndEarmarkAndTodayInPager
     * @Description : 
     * @Params      : [earmark, deviceCode, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8       
     * @Time        : 20:37
     */
    @Override
    public Result listEartagFlowByDeviceCodeAndEarmarkAndTodayInPager(String earmark, String deviceCode, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagDataEntity> sf = (Specification<EartagDataEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //earmark
            if (null != earmark) {
                predList.add(criteriaBuilder.equal(root.get("earmark").as(String.class), earmark));
            }
            //
            if (null != deviceCode) {
                predList.add(criteriaBuilder.equal(root.get("device").as(String.class), deviceCode));
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, eartagDataRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listEartagFlowByDeviceCodeAndEarmarkAndDateInPager
     * @Description : 
     * @Params      : [earmark, deviceCode, addDate, pageNo, pageSize]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/9       
     * @Time        : 22:14
     */
    @Override
    public Result listEartagFlowByDeviceCodeAndEarmarkAndDateInPager(String earmark, String deviceCode, String addDate, Integer pageNo, Integer pageSize) {
        //
        Specification<EartagDataEntity> sf = (Specification<EartagDataEntity>) (root, criteriaQuery, criteriaBuilder) -> {
            //
            List<Predicate> predList = new ArrayList<>();
            //earmark
            if (null != earmark && earmark.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("earmark").as(String.class), earmark));
            }
            //
            if (null != deviceCode && deviceCode.trim().length()>0) {
                predList.add(criteriaBuilder.equal(root.get("device").as(String.class), deviceCode));
            }
            //
            if (null != addDate && addDate.trim().length()>0) {
                try {
                    predList.add(criteriaBuilder.equal(root.get("createDate").as(Date.class), new Date(new DateUtil().parseDateTextToLong(addDate))));
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            Predicate[] pred = new Predicate[predList.size()];
            Predicate and = criteriaBuilder.and(predList.toArray(pred));
            criteriaQuery.where(and);
            //
            List<Order> orders = new ArrayList<>();
            orders.add(criteriaBuilder.desc(root.get("id")));
            return criteriaQuery.orderBy(orders).getRestriction();
        };
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return new Result(ResultCode.SUCCESS, eartagDataRepo.findAll(sf, pageable));
    }

    /**
     * @Method      : listEartagOnlineCountByEarmarkAndDate
     * @Description : 显示耳标在线统计
     * @Params      : [earmark, pastDays]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/8       
     * @Time        : 18:30
     */
    @Override
    public Result listEartagOnlineCountByEarmarkAndDate(String earmark, Integer pastDays) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        String startDateText = du.getPastDate(pastDays);
        log.info("<listEartagOnlineCountByEarmarkAndDate> startDateText>>"+startDateText);
        log.info("<listEartagOnlineCountByEarmarkAndDate> todayDateText>>"+todayDateText);
        //读取耳标在线统计记录
        List<EartagEartagOnlineEntity> eartagOnlineEntityList = eartagOnlineRepo.getEartagOnlineByEarmarkAndDate(earmark,startDateText,todayDateText);
        if(eartagOnlineEntityList.size()>0){
            JSONArray dataJa = new JSONArray();
            for(EartagEartagOnlineEntity eartagOnlineEntity:eartagOnlineEntityList){
                JSONObject eartagOnlineJo = JSONUtil.convertEntityToJSONObject(eartagOnlineEntity);
                eartagOnlineJo.put("firstTime",du.formatDatetimeText(eartagOnlineJo.getDate("firstTime")));
                eartagOnlineJo.put("lastTime",du.formatDatetimeText(eartagOnlineJo.getDate("lastTime")));
                eartagOnlineJo.put("addDate",du.formatDateText(eartagOnlineJo.getDate("addDate")));
                dataJa.add(eartagOnlineJo);
            }
            return new Result(ResultCode.SUCCESS,dataJa);
        }else{
            return new Result(BizConst.CODE_EARTAG_ONLINE_NO_EXIST,BizConst.MSG_EARTAG_ONLINE_NO_EXIST,false);
        }
    }

    /**
     * @Method      : listEartagFlowByDeviceCodeAndEarmarkAndToday
     * @Description : 根据设备编码、耳标号和当天日期读取耳标流水数据
     * @Params      : [earmark, deviceCode]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/7       
     * @Time        : 18:46
     */
    @Override
    public Result listEartagFlowByDeviceCodeAndEarmarkAndToday(String earmark, String deviceCode) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        //
        List<EartagDataEntity> eartagDataEntityList = eartagDataRepo.getEartagFlowByDeviceCodeAndEarmarkAndToday(deviceCode,earmark,todayDateText);
        if(eartagDataEntityList.size()>0){
            return new Result(ResultCode.SUCCESS,JSONObject.toJSONString(eartagDataEntityList));
        }else{
            return new Result(BizConst.CODE_EARTAG_FLOW_NO_EXIST,BizConst.MSG_EARTAG_FLOW_NO_EXIST,false);
        }
    }

    /**
     * @Method      : listEartagFlowByEarmarkAndToday
     * @Description : 根据耳标号读取今天所有流水
     * @Params      : [earmark]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/7       
     * @Time        : 18:41
     */
    @Override
    public Result listEartagFlowByEarmarkAndToday(String earmark) throws ParseException {
        DateUtil du = new DateUtil();
        String todayDateText = du.getTodayDateText();
        //
        List<EartagDataEntity> eartagDataEntityList = eartagDataRepo.getEartagFlowByEarmarkAndToday(earmark,todayDateText);
        if(eartagDataEntityList.size()>0){
            return new Result(ResultCode.SUCCESS,JSONObject.toJSONString(eartagDataEntityList));
        }else{
            return new Result(BizConst.CODE_EARTAG_FLOW_NO_EXIST,BizConst.MSG_EARTAG_FLOW_NO_EXIST,false);
        }
    }

    /**
     * @Method      : getEartagRegisterByEarmark
     * @Description : 
     * @Params      : [earmark]
     * @Return      : com.huimv.eartag2.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2022/3/7       
     * @Time        : 18:41
     */
    @Override
    public Result getEartagRegisterByEarmark(String earmark) {
        //
        EartagEartagRegisterEntity eartagRegisterEntity = eartagRegisterRepo.getEartagByEarmark(earmark);
        if(eartagRegisterEntity != null){
            return new Result(ResultCode.SUCCESS, JSONObject.toJSONString(eartagRegisterEntity));
        }else{
            return new Result(BizConst.CODE_EARTAG_REGISTER_NO_EXIST,BizConst.MSG_EARTAG_REGISTER_NO_EXIST,false);
        }
    }


}
